package com.burtbeckwith.gae.guestbook.controller

import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Controller
import org.springframework.ui.Model
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestMethod
import org.springframework.web.bind.annotation.RequestParam

import com.burtbeckwith.gae.guestbook.domain.AddressBookEntry
import com.burtbeckwith.gae.guestbook.service.AddressBookService

@Controller
class AddressBookController {

	private final AddressBookService _addressBookService

	@Autowired
	AddressBookController(AddressBookService addressBookService) {
		_addressBookService = addressBookService
	}

	@RequestMapping(value="/addressbook/list.action", method=RequestMethod.GET)
	String list(@RequestParam(value='page', required=false) String keyOffset,
			@RequestParam(value='startIndex', required=false) String indexString,
			@RequestParam(value='lastNameSelected', required=false) String lastNameSelected,
			@RequestParam(value='stateSelected', required=false) String stateSelected,
			Model model) {

		Long keyOffsetId = keyOffset ? Long.decode(keyOffset) : null
		if (!keyOffset) {
			keyOffset = ''
		}

		if (!indexString) {
			indexString = '0'
		}
		int indexOffset = indexString.toInteger()

		if (!stateSelected) {
			stateSelected = ''
		}

		List<AddressBookEntry> entries = new ArrayList<AddressBookEntry>(
				_addressBookService.getPage(keyOffsetId, indexOffset, lastNameSelected, stateSelected))
		AddressBookEntry lastEntry = null
		if (entries.size() > 3) {
			lastEntry = entries[3]
			entries.remove(3)
		}

		model.addAttribute('lastNameSelected', lastNameSelected)
		model.addAttribute('stateSelected', stateSelected)
		model.addAttribute('entries', entries)
		model.addAttribute('lastEntry', lastEntry)
		model.addAttribute('indexOffset', indexOffset)

		return 'addressbook'
	}

	@RequestMapping(value="/addressbook/addEntry.action", method=RequestMethod.POST)
	String addEntry(
			@RequestParam('lastName') String lastName,
			@RequestParam('firstName') String firstName,
			@RequestParam('city') String city,
			@RequestParam('state') String state,
			@RequestParam('phoneNumber') String phoneNumber) {

		_addressBookService.insertNew(firstName, lastName, city,
				state, phoneNumber)

		return 'redirect:/addressbook/list.action'
	}
}
