package com.burtbeckwith.gae.guestbook.controller

import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Controller
import org.springframework.ui.Model
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestMethod
import org.springframework.web.bind.annotation.RequestParam

import com.burtbeckwith.gae.guestbook.domain.NamedCounter
import com.burtbeckwith.gae.guestbook.service.NamedCounterService

@Controller
class NamedCounterController {

	private final NamedCounterService _namedCounterService

	@Autowired
	NamedCounterController(NamedCounterService namedCounterService) {
		_namedCounterService = namedCounterService
	}

	@RequestMapping(value="/namedcounter/show.action", method=RequestMethod.GET)
	String setupForm(@RequestParam(value='name', required=false) String name,
			Model model) {

		if (!name) {
			name = 'default'
		}

		NamedCounter counter = _namedCounterService.getByName(name)

		model.addAttribute('name', name)
		model.addAttribute('counter', counter)

		return 'namedcounter'
	}

	@RequestMapping(value="/namedcounter/update.action", method=RequestMethod.POST)
	String update(@RequestParam('submit') String action, @RequestParam('name') String name,
			@RequestParam(value='delta', required=false) Integer delta) {

		if ('Reset'.equals(action)) {
			_namedCounterService.reset(name)
		}
		else if ('Offset'.equals(action)) {
			_namedCounterService.addAndGet(name, delta)
		}
		else if ('Get'.equals(action)) {
			// Do nothing. The redirect will handle this.
		}

		return "redirect:/namedcounter/show.action?name=$name"
	}
}
