package com.burtbeckwith.gae.guestbook.service

import javax.jdo.PersistenceManager

import com.burtbeckwith.gae.guestbook.domain.*

class AddressBookService {

	private static final int ENTITIES_PER_PAGE = 3

	String insertNew(String firstName, String lastName, String city,
			String state, String phoneNumber) {

		AddressBookEntry entry = new AddressBookEntry(
				personalInfo: new PersonalInfo(firstName, lastName),
				addressInfo: new AddressInfo(city, state),
				contactInfo: new ContactInfo(phoneNumber))
		entry.save()

		return entry.id.toString()
	}

	List<AddressBookEntry> getPage(Long keyOffset, int indexOffset,
			String lastName, String state) {

		AddressBookEntry.withPersistenceManager { pm ->

			def query = pm.newQuery(AddressBookEntry)
			query.declareParameters('Long keyOffset, String lastNameSelected, String stateSelected')

			StringBuilder filter = new StringBuilder()

			if (keyOffset) {
				if (filter.length()) {
					filter.append(' && ')
				}
				filter.append('id >= keyOffset')
			}

			if (lastName) {
				if (filter.length()) {
					filter.append(' && ')
				}
				filter.append('personalInfo.lastName == lastNameSelected')
			}

			if (state) {
				if (filter.length()) {
					filter.append(' && ')
				}
				filter.append('addressInfo.state == stateSelected')
			}

			if (filter.length()) {
				query.filter = filter.toString()
			}

			query.setRange(indexOffset, indexOffset + ENTITIES_PER_PAGE + 1)

			return query.execute(keyOffset, lastName, state)
		}
	}
}
