package com.burtbeckwith.gae.guestbook.service

import com.google.appengine.api.datastore.Key
import com.google.appengine.api.datastore.KeyFactory

import com.burtbeckwith.gae.guestbook.domain.Friend

class FriendService {

	void addFriendTo(String lastName, String firstName,
			String friendLastName, String friendFirstName) {

		String meKey = generateKey(lastName, firstName)
		String otherKey = generateKey(friendLastName, friendFirstName)

		Friend.withTransaction { pm ->
			// First create the friend if he doesn't already exist.
			Friend other = Friend.get(otherKey) ?: new Friend(friendLastName, friendFirstName)
			other.friendKeys << meKey
			if (!other.isJdoPersistent()) {
				other.save()
			}
		}

		Friend.withTransaction { pm ->
			// Then add a reference to the friend to my list.
			Friend me = Friend.get(meKey) ?: new Friend(lastName, firstName)
			me.friendKeys << otherKey
			if (!me.isJdoPersistent()) {
				me.save()
			}
		}
	}

	List<Friend> getFriendsOf(String lastName, String firstName) {
		Friend.withPersistenceManager { pm ->
			String myKey = generateKey(lastName, firstName)
			def query = pm.newQuery(Friend)
			query.declareParameters('String myKey')
			query.setFilter('friendKeys == myKey')
			query.setOrdering('lastName ASC, firstName ASC')
			return query.execute(myKey)
		}
	}

	private String generateKey(String lastName, String firstName) {
		Key key = KeyFactory.createKey(Friend.simpleName, lastName + ', ' + firstName)
		return KeyFactory.keyToString(key)
	}
}
