package com.burtbeckwith.gae.framework;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Abstract base class for filters.
 *
 * @author <a href='mailto:burt@burtbeckwith.com'>Burt Beckwith</a>
 */
public abstract class AbstractFilter implements Filter {

	/**
	 * {@inheritDoc}
	 * @see javax.servlet.Filter#doFilter(javax.servlet.ServletRequest,
	 * 	javax.servlet.ServletResponse, javax.servlet.FilterChain)
	 */
	public final void doFilter(final ServletRequest request, final ServletResponse response,
			FilterChain chain) throws IOException, ServletException {

		doFilterHttp((HttpServletRequest)request, (HttpServletResponse)response, chain);
	}

	protected abstract void doFilterHttp(HttpServletRequest request, HttpServletResponse response,
			FilterChain chain) throws IOException, ServletException;

	/**
	 * {@inheritDoc}
	 * @see javax.servlet.Filter#init(javax.servlet.FilterConfig)
	 */
	public void init(final FilterConfig filterConfig) throws ServletException {
		// override as necessary
	}

	/**
	 * {@inheritDoc}
	 * @see javax.servlet.Filter#destroy()
	 */
	public void destroy() {
		// override as necessary
	}
}
