package com.burtbeckwith.gae.framework;

import groovy.lang.GString;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.codehaus.groovy.runtime.DefaultGroovyMethods;

/**
 * 'params' class - a simplified version of GrailsParameterMap.
 *
 * @author <a href='mailto:burt@burtbeckwith.com'>Burt Beckwith</a>
 */
@SuppressWarnings("unchecked")
public class ParameterMap implements Map {

	private final Map<String, Object> _parameterMap = new HashMap<String, Object>();
	private final HttpServletRequest _request;

	public ParameterMap(HttpServletRequest request) {
		_request = request;
		Map requestMap = new LinkedHashMap(request.getParameterMap());
		for (String key : (Set<String>)requestMap.keySet()) {
			Object paramValue = getParameterValue(requestMap, key);
			_parameterMap.put(key, paramValue);
			processNestedKeys(requestMap, key, key, _parameterMap);
		}
	}

	public ParameterMap(Map<String, Object> values, HttpServletRequest request) {
		_request = request;
		_parameterMap.putAll(values);
	}

	private Object getParameterValue(Map requestMap, String key) {
		Object paramValue = requestMap.get(key);
		if (paramValue instanceof String[]) {
			String[] multiParams = (String[])paramValue;
			if (multiParams.length == 1) {
				paramValue = multiParams[0];
			}
		}
		return paramValue;
	}

	private void processNestedKeys(Map requestMap, String key, String nestedKey, Map nestedLevel) {
		final int nestedIndex = nestedKey.indexOf('.');
		if (nestedIndex > -1) {
			String nestedPrefix = nestedKey.substring(0, nestedIndex);

			Object prefixValue = nestedLevel.get(nestedPrefix);
			if (prefixValue == null) {
				prefixValue = new ParameterMap(new HashMap<String, Object>(), _request);
				nestedLevel.put(nestedPrefix, prefixValue);
			}

			if (prefixValue instanceof Map) {
				Map nestedMap = (Map)prefixValue;
				if (nestedIndex < nestedKey.length() - 1) {
					String remainderOfKey = nestedKey.substring(nestedIndex + 1, nestedKey.length());
					nestedMap.put(remainderOfKey,getParameterValue(requestMap, key) );
					if (remainderOfKey.indexOf('.') > -1) {
						processNestedKeys(requestMap, key, remainderOfKey, nestedMap);
					}
				}
			}
		}
	}

	public int size() {
		return _parameterMap.size();
	}

	public boolean isEmpty() {
		return _parameterMap.isEmpty();
	}

	public boolean containsKey(Object key) {
		return _parameterMap.containsKey(key);
	}

	public boolean containsValue(Object value) {
		return _parameterMap.containsValue(value);
	}

	public Object get(Object key) {
		Object value = _parameterMap.get(key);
		if (value instanceof String []) {
			String[] valueArray = (String[])_parameterMap.get(key);
			if (valueArray == null){
				return null;
			}

			if (valueArray.length == 1) {
				return valueArray[0];
			}
		}

		return value;

	}

	public Object put(Object key, Object value) {
		if (value instanceof GString) {
			value = value.toString();
		}
		return _parameterMap.put(key.toString(), value);
	}

	public Object remove(Object key) {
		return _parameterMap.remove(key);
	}

	public void putAll(Map map) {
		for (Map.Entry entry : (Set<Map.Entry>)map.entrySet()) {
			put(entry.getKey(), entry.getValue());
		}
	}

	public void clear() {
		_parameterMap.clear();
	}

	public Set keySet() {
		return _parameterMap.keySet();
	}

	public Collection values() {
		return _parameterMap.values();
	}

	public Set entrySet() {
		return _parameterMap.entrySet();
	}

	@Override
	public String toString() {
		return DefaultGroovyMethods.inspect(_parameterMap);
	}
}
